from typing import Any
from urllib.parse import urljoin
from airflow.providers.http.hooks.http import HttpHook
from airflow import settings
from airflow.models.connection import Connection
from requests.sessions import session
from datacat.DataCatConnection import DataCatConnection, DataCatalogEntry
from requests import Session, Request
from typing import Optional, Dict
import json

from datacat.BearerAuth import BearerAuth

def get_connection_with_secrets_from_entry(data: Dict[str, Any], secrets: Dict[str, str] , datacat_type: str, oid: str) -> Connection:
    """returns an aiflow connection from the data provided in the datacat entry and the secrets."""
    conn = get_connection_from_entry(data, datacat_type, oid)
    conn.password = secrets['password']
    conn.login = secrets['login']
    # add all remaining secrets to extra
    extra = conn.extra
    for key in secrets.keys():
        if key in ['password', 'login']:
            continue
        extra[key] = secrets[key]

    conn.extra = extra
    return conn


def get_connection_from_entry(data: Dict[str, Any], datacat_type: str, oid: str) -> Connection:
    """returns an aiflow connection from the data provided in the datacat entry."""
    conn_type = data['metadata']['conn_type']
    host = data['metadata']['host']
    port = data['metadata']['port']
    schema = data['metadata']['schema']
    conn_id = f"{datacat_type}/{oid}-connection"
    # set all remaining metadata as extra
    extra = {}
    for key in data['metadata'].keys():
        if key in ['conn_type', 'host', 'port', 'schema']:
            continue
        extra[key] = data['metadata'][key]

    return Connection(
        conn_id=conn_id,
        conn_type=conn_type,
        host=host,
        port=port,
        description="Automatically generated Connection for the given entry.",
        schema=schema,
        extra=extra
    )


class DataCatalogHook(HttpHook):
    connection: DataCatConnection = None

    def __init__(self, http_conn_id: str = 'datacatalog') -> None:
        super().__init__(method="GET", http_conn_id=http_conn_id)
        conn = super().get_conn()
        self.connection = DataCatConnection(catalog_url=self.base_url, username=conn.auth.username, password=conn.auth.password)

    def get_conn(self, headers: Optional[Dict[Any, Any]] = None) -> Session:
        conn = super().get_conn(headers=headers)
        self.connection = DataCatConnection(catalog_url=self.base_url, username=conn.auth.username, password=conn.auth.password)
        conn.auth = BearerAuth(self.connection.get_token())
        return conn

    def get_object(self, datacat_type: str, oid: str):
        return self.connection.get_object(datacat_type, oid)

    def create_object(self, datacat_type: str, object: DataCatalogEntry):
        return self.connection.create_object(datacat_type, object)

    def list_type(self, datacat_type: str):
        return self.connection.list_type(datacat_type)

    def create_get_object_connection(self, datacat_type: str, oid: str):
        object = self.get_object(datacat_type=datacat_type, oid=oid)
        # TODO decide some factors, such as connection type
        conn = get_connection_from_entry(object, datacat_type, oid)
        
        session = settings.Session()

        conn_name = session.query(Connection).filter(Connection.conn_id == conn.conn_id).first()
        if str(conn_name) != str (conn.conn_id):
            session.add(conn)
            session.commit()
